using ComputersShopContracts.BindingModels;
using ComputersShopContracts.SearchModels;
using ComputersShopContracts.StoragesContracts;
using ComputersShopContracts.ViewModels;
using ComputersShopDataModels;
using ComputersShopFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputersShopFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton _source;
        public ShopStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }
        public List<ShopViewModel> GetFullList()
        {
            return _source.Shops.Select(x => x.GetViewModel).ToList();
        }
        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name))
            {
                return new();
            }
            return _source.Shops.Where(x => x.ShopName.Contains(model.Name)).Select(x => x.GetViewModel).ToList();
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
            {
                return null;
            }
            return _source.Shops.FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && x.ShopName == model.Name) || (model.Id.HasValue && x.Id == model.Id)) ?.GetViewModel;
        }
        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = _source.Shops.Count > 0 ? _source.Shops.Max(x => x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            _source.Shops.Add(newShop);
            _source.SaveShops();
            return newShop.GetViewModel;
        }
        public ShopViewModel? Update(ShopBindingModel model)
        {
            var component = _source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            _source.SaveShops();
            return component.GetViewModel;
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var element = _source.Shops.FirstOrDefault(x => x.Id ==
           model.Id);
            if (element != null)
            {
                _source.Shops.Remove(element);
                _source.SaveShops();
                return element.GetViewModel;
            }
            return null;
        }

        private bool CheckAvailability(int ComputerId, int count)
        {
            count -= _source.Shops.Select(x => x.ShopComputers.Select(y => (y.Value.Item1.Id == ComputerId ? y.Value.Item2 : 0)).Sum()).Sum();
            return count <= 0;
        }

        public bool SellComputers(IComputerModel model, int count)
        {
            var NeededComputer = _source.Computers.FirstOrDefault(x => x.Id == model.Id);

            if (NeededComputer == null || !CheckAvailability(NeededComputer.Id, count))
            {
                return false;
            }

            for (int i = 0; i < _source.Shops.Count; i++)
            {
                var shop = _source.Shops[i];
                var Computers = shop.ShopComputers;

                foreach (var computer in Computers.Where(x => x.Value.Item1.Id == NeededComputer.Id))
                {
                    var min = Math.Min(computer.Value.Item2, count);
                    Computers[computer.Value.Item1.Id] = (computer.Value.Item1, computer.Value.Item2 - min);
                    count -= min;

                    if (count <= 0)
                    {
                        break;
                    }
                }

                shop.Update(new ShopBindingModel
                {
                    Id = shop.Id,
                    ShopName = shop.ShopName,
                    Address = shop.Address,
                    DateOpen = shop.DateOpen,
                    MaxCapacity = shop.MaxCapacity,
                    ShopComputers = Computers
                });
            }
            _source.SaveShops();
            return true;
        }
    }
}
